@@ -12,7 +12,7 @@ from zope.interface import implementer
 from zope.interface.verify import verifyObject
 
 from twisted.python import reflect, failure
-from twisted.python.compat import _PY3
+from twisted.python.compat import _PY3, unichr
 from twisted.python.filepath import FilePath
 from twisted.trial import unittest
 from twisted.internet import reactor
@@ -33,9 +33,9 @@ class ResourceTests(unittest.TestCase):
 
 class SimpleResource(resource.Resource):
     """
-    @ivar _contentType: C{None} or a C{str} giving the value of the
+    @ivar _contentType: L{None} or a C{str} giving the value of the
         I{Content-Type} header in the response this resource will render.  If it
-        is C{None}, no I{Content-Type} header will be set in the response.
+        is L{None}, no I{Content-Type} header will be set in the response.
     """
     def __init__(self, contentType=None):
         resource.Resource.__init__(self)
@@ -136,6 +136,32 @@ class SiteTest(unittest.TestCase):
         self.assertIsInstance(session.uid, bytes)
 
 
+    def test_sessionUIDGeneration(self):
+        """
+        L{site.getSession} generates L{Session} objects with distinct UIDs from
+        a secure source of entropy.
+        """
+        site = server.Site(resource.Resource())
+        # Ensure that we _would_ use the unpredictable random source if the
+        # test didn't stub it.
+        self.assertIdentical(site._entropy, os.urandom)
+
+        def predictableEntropy(n):
+            predictableEntropy.x += 1
+            return (unichr(predictableEntropy.x) * n).encode("charmap")
+        predictableEntropy.x = 0
+        self.patch(site, "_entropy", predictableEntropy)
+        a = self.getAutoExpiringSession(site)
+        b = self.getAutoExpiringSession(site)
+        self.assertEqual(a.uid, b"01" * 0x20)
+        self.assertEqual(b.uid, b"02" * 0x20)
+        # This functionality is silly (the value is no longer used in session
+        # generation), but 'counter' was a public attribute since time
+        # immemorial so we should make sure if anyone was using it to get site
+        # metrics or something it keeps working.
+        self.assertEqual(site.counter, 2)
+
+
     def test_getSessionExistent(self):
         """
         L{site.getSession} gets a previously generated session, by its unique
@@ -554,8 +580,10 @@ class RequestTests(unittest.TestCase):
         fail = failure.Failure(Exception("Oh no!"))
         request.processingFailed(fail)
 
-        self.assertNotIn(b"Oh no!", request.transport.getvalue())
-        self.assertIn(b"Processing Failed", request.transport.getvalue())
+        self.assertNotIn(b"Oh no!", request.transport.written.getvalue())
+        self.assertIn(
+            b"Processing Failed", request.transport.written.getvalue()
+        )
 
         # Since we didn't "handle" the exception, flush it to prevent a test
         # failure
@@ -574,7 +602,7 @@ class RequestTests(unittest.TestCase):
         fail = failure.Failure(Exception("Oh no!"))
         request.processingFailed(fail)
 
-        self.assertIn(b"Oh no!", request.transport.getvalue())
+        self.assertIn(b"Oh no!", request.transport.written.getvalue())
 
         # Since we didn't "handle" the exception, flush it to prevent a test
         # failure
@@ -594,13 +622,109 @@ class RequestTests(unittest.TestCase):
         fail = failure.Failure(Exception(u"\u2603"))
         request.processingFailed(fail)
 
-        self.assertIn(b"&#9731;", request.transport.getvalue())
+        self.assertIn(b"&#9731;", request.transport.written.getvalue())
 
         # Since we didn't "handle" the exception, flush it to prevent a test
         # failure
         self.assertEqual(1, len(self.flushLoggedErrors()))
 
 
+    def test_sessionDifferentFromSecureSession(self):
+        """
+        L{Request.session} and L{Request.secure_session} should be two separate
+        sessions with unique ids and different cookies.
+        """
+        d = DummyChannel()
+        d.transport = DummyChannel.SSL()
+        request = server.Request(d, 1)
+        request.site = server.Site(resource.Resource())
+        request.sitepath = []
+        secureSession = request.getSession()
+        self.assertIsNotNone(secureSession)
+        self.addCleanup(secureSession.expire)
+        self.assertEqual(request.cookies[0].split(b"=")[0],
+                         b"TWISTED_SECURE_SESSION")
+        session = request.getSession(forceNotSecure=True)
+        self.assertIsNotNone(session)
+        self.assertEqual(request.cookies[1].split(b"=")[0], b"TWISTED_SESSION")
+        self.addCleanup(session.expire)
+        self.assertNotEqual(session.uid, secureSession.uid)
+
+
+    def test_sessionAttribute(self):
+        """
+        On a L{Request}, the C{session} attribute retrieves the associated
+        L{Session} only if it has been initialized.  If the request is secure,
+        it retrieves the secure session.
+        """
+        site = server.Site(resource.Resource())
+        d = DummyChannel()
+        d.transport = DummyChannel.SSL()
+        request = server.Request(d, 1)
+        request.site = site
+        request.sitepath = []
+        self.assertIs(request.session, None)
+        insecureSession = request.getSession(forceNotSecure=True)
+        self.addCleanup(insecureSession.expire)
+        self.assertIs(request.session, None)
+        secureSession = request.getSession()
+        self.addCleanup(secureSession.expire)
+        self.assertIsNot(secureSession, None)
+        self.assertIsNot(secureSession, insecureSession)
+        self.assertIs(request.session, secureSession)
+
+
+    def test_sessionCaching(self):
+        """
+        L{Request.getSession} creates the session object only once per request;
+        if it is called twice it returns the identical result.
+        """
+        site = server.Site(resource.Resource())
+        d = DummyChannel()
+        request = server.Request(d, 1)
+        request.site = site
+        request.sitepath = []
+        session1 = request.getSession()
+        self.addCleanup(session1.expire)
+        session2 = request.getSession()
+        self.assertIs(session1, session2)
+
+
+    def test_retrieveExistingSession(self):
+        """
+        L{Request.getSession} retrieves an existing session if the relevant
+        cookie is set in the incoming request.
+        """
+        site = server.Site(resource.Resource())
+        d = DummyChannel()
+        request = server.Request(d, 1)
+        request.site = site
+        request.sitepath = []
+        mySession = server.Session(b"special-id", site)
+        site.sessions[mySession.uid] = mySession
+        request.received_cookies[b'TWISTED_SESSION'] = mySession.uid
+        self.assertIs(request.getSession(), mySession)
+
+
+    def test_retrieveNonExistentSession(self):
+        """
+        L{Request.getSession} generates a new session if the relevant cookie is
+        set in the incoming request.
+        """
+        site = server.Site(resource.Resource())
+        d = DummyChannel()
+        request = server.Request(d, 1)
+        request.site = site
+        request.sitepath = []
+        request.received_cookies[b'TWISTED_SESSION'] = b"does-not-exist"
+        session = request.getSession()
+        self.assertIsNotNone(session)
+        self.addCleanup(session.expire)
+        self.assertTrue(request.cookies[0].startswith(b'TWISTED_SESSION='))
+        # It should be a new session ID.
+        self.assertNotIn(b"does-not-exist", request.cookies[0])
+
+
 
 class GzipEncoderTests(unittest.TestCase):
 
@@ -823,11 +947,15 @@ class NewRenderTests(unittest.TestCase):
     def testGoodMethods(self):
         req = self._getReq()
         req.requestReceived(b'GET', b'/newrender', b'HTTP/1.0')
-        self.assertEqual(req.transport.getvalue().splitlines()[-1], b'hi hi')
+        self.assertEqual(
+            req.transport.written.getvalue().splitlines()[-1], b'hi hi'
+        )
 
         req = self._getReq()
         req.requestReceived(b'HEH', b'/newrender', b'HTTP/1.0')
-        self.assertEqual(req.transport.getvalue().splitlines()[-1], b'ho ho')
+        self.assertEqual(
+            req.transport.written.getvalue().splitlines()[-1], b'ho ho'
+        )
 
     def testBadMethods(self):
         req = self._getReq()
@@ -855,7 +983,9 @@ class NewRenderTests(unittest.TestCase):
         req = self._getReq()
         req.requestReceived(b'HEAD', b'/newrender', b'HTTP/1.0')
         self.assertEqual(req.code, 200)
-        self.assertEqual(-1, req.transport.getvalue().find(b'hi hi'))
+        self.assertEqual(
+            -1, req.transport.written.getvalue().find(b'hi hi')
+        )
 
 
     def test_unsupportedHead(self):
@@ -866,7 +996,7 @@ class NewRenderTests(unittest.TestCase):
         resource = HeadlessResource()
         req = self._getReq(resource)
         req.requestReceived(b"HEAD", b"/newrender", b"HTTP/1.0")
-        headers, body = req.transport.getvalue().split(b'\r\n\r\n')
+        headers, body = req.transport.written.getvalue().split(b'\r\n\r\n')
         self.assertEqual(req.code, 200)
         self.assertEqual(body, b'')
 
@@ -887,7 +1017,7 @@ class NewRenderTests(unittest.TestCase):
 
         request.requestReceived(b"GET", b"/newrender", b"HTTP/1.0")
 
-        headers, body = request.transport.getvalue().split(b'\r\n\r\n')
+        headers, body = request.transport.written.getvalue().split(b'\r\n\r\n')
         self.assertEqual(request.code, 500)
         expected = [
             '',
@@ -988,7 +1118,7 @@ class AllowedMethodsTests(unittest.TestCase):
         req.requestReceived(b'POST', b'/gettableresource?'
                             b'value=<script>bad', b'HTTP/1.0')
         self.assertEqual(req.code, 405)
-        renderedPage = req.transport.getvalue()
+        renderedPage = req.transport.written.getvalue()
         self.assertNotIn(b"<script>bad", renderedPage)
         self.assertIn(b'&lt;script&gt;bad', renderedPage)
 
@@ -1003,7 +1133,7 @@ class AllowedMethodsTests(unittest.TestCase):
         req = self._getReq()
         req.requestReceived(b'<style>bad', b'/gettableresource', b'HTTP/1.0')
         self.assertEqual(req.code, 501)
-        renderedPage = req.transport.getvalue()
+        renderedPage = req.transport.written.getvalue()
         self.assertNotIn(b"<style>bad", renderedPage)
         self.assertIn(b'&lt;style&gt;bad', renderedPage)
 
